use crate::{ExternalReferenceType, VecExternRef};
use alloc::boxed::Box;
use alloc::collections::BTreeMap;
use alloc::vec::Vec;
use core::marker::PhantomData;

struct ImmutableListInner<T: Clone> {
    extern_ref: VecExternRef<T>,
    resource_type: PhantomData<T>,
    values: spin::RwLock<Vec<Option<Option<T>>>>,
}

#[derive(Clone)]
pub struct ImmutableList<T>
where
    T: Clone,
    T: ExternalReferenceType,
{
    inner: *const ImmutableListInner<T>,
}

impl<T> ImmutableList<T>
where
    T: Clone,
    T: ExternalReferenceType,
{
    fn new(extern_ref: VecExternRef<T>) -> Self {
        let mut values = Vec::new();
        values.resize(extern_ref.len() as usize, None);
        let inner = Box::new(ImmutableListInner {
            extern_ref,
            resource_type: Default::default(),
            values: spin::RwLock::new(values),
        });
        let inner_ptr = Box::into_raw(inner);
        ImmutableList {
            inner: inner_ptr as *const ImmutableListInner<T>,
        }
    }

    pub(crate) fn from_ref(extern_ref: VecExternRef<T>) -> Self {
        unsafe {
            let existing = CACHE.get(&extern_ref.get_internal_index());
            if let Some(v) = existing {
                let inner = *v as *const ImmutableListInner<T>;
                ImmutableList { inner }
            } else {
                let v = Self::new(extern_ref);
                CACHE.insert(extern_ref.get_internal_index(), v.inner as *const u8);
                v
            }
        }
    }

    pub fn get(&self, index: u32) -> Option<T> {
        unsafe {
            let inner = self.inner.as_ref().unwrap();
            {
                let rg = inner.values.read();
                let v = rg.get(index as usize).unwrap();
                if let Some(v) = v {
                    return v.clone();
                }
            }
            let r = inner.extern_ref.at(index);
            let value = r.get_value();
            let mut wg = inner.values.write();
            wg[index as usize] = Some(value);
            wg[index as usize].as_ref().unwrap().clone()
        }
    }
}

static mut CACHE: BTreeMap<u32, *const u8> = BTreeMap::new();
